import bpy

from .. import utils


class THOMAS_RIG_UPDATE_RIG(bpy.types.Operator):
    bl_idname = "thomasriglegacy.update_rig"
    bl_label = "update"
    bl_options = {'REGISTER', 'UNDO'}

    rig = None
    rig_version = None
    ext_version = None
    
    def execute(self, context):
        self.rig = utils.get_rig()
        self.rig_version = self.rig.get("version", None)
        self.ext_version = utils.get_ext_version()
        
        if self.rig_version is None or tuple(self.rig_version) < (0, 2, 0):
            self.update_to_0_2_0()

        # set rig version to newest
        self.rig["rig_version"] = self.ext_version
        self.rig.data.update_tag()
        bpy.context.view_layer.update()
        self.report({"INFO"}, "Rig updated to v" + '.'.join(map(str, self.ext_version)))
        return {'FINISHED'}
    
    def update_to_0_2_0(self):
        # hide eyes boolean
        for object in self.rig.children_recursive:
            if "Eyes" in object.name:
                object.modifiers["Boolean"].show_viewport = False
                break


        # Access Jaw constraint influence driver
        constraint = self.rig.pose.bones["Jaw"].constraints["LowEyes"]
        driver = constraint.driver_add("influence").driver

        # Add a new driver variable
        new_var = driver.variables.new()
        new_var.name = "low"
        new_var.type = "SINGLE_PROP"

        # Configure the new variable
        target = new_var.targets[0]
        target.id_type = "OBJECT"
        target.id = self.rig
        target.data_path = 'pose.bones["Eyes_UP_DOWN"].location[1]'

        # Set the driver expression
        driver.expression = "((1/(8*location-1)+1) *-low*5 if location<0 else 0)"


        # add chibi mode | property
        misc_props = self.rig.pose.bones["Misc_Properties"]
        misc_props["Chibi"] = False

        def add_chibi(target_bone_name, scale_factors):
            constraint = self.rig.pose.bones[target_bone_name].constraints.new(type="TRANSFORM")
            constraint.name = 'Chibi'
            constraint.target = self.rig
            constraint.subtarget = f'{self.rig.pose.bones["StaticBone"].name}'
            constraint.target_space = "LOCAL"
            constraint.owner_space = "LOCAL"
            constraint.map_to = "SCALE"
            constraint.to_min_x_scale = scale_factors
            constraint.to_max_x_scale = scale_factors
            constraint.to_min_y_scale = scale_factors
            constraint.to_max_y_scale = scale_factors
            constraint.to_min_z_scale = scale_factors
            constraint.to_max_z_scale = scale_factors
            constraint.mix_mode_scale = 'MULTIPLY'

            # add driver 1
            driver = constraint.driver_add("enabled").driver

            new_var = driver.variables.new()
            new_var.name = "chibi"
            new_var.type = "SINGLE_PROP"
            target = new_var.targets[0]
            target.id_type = "OBJECT"   
            target.id = self.rig
            target.data_path = 'pose.bones["Misc_Properties"]["Chibi"]'
            driver.expression = "chibi"

        add_chibi("Root", 2/3)
        add_chibi("Head", 1.25)